from eyja.hubs.config_hub import ConfigHub
from eyja.utils import random_string, load_model

from eyja_fastapi.models import RefreshToken

from .statuses import RefreshTokenStatuses
from .constants import RefreshTokenConstants


class RefreshTokenOperator:
    statuses = RefreshTokenStatuses
    constants = RefreshTokenConstants

    @classmethod
    async def create_token(cls, user, status=RefreshTokenStatuses.ACTIVE):
        refresh_token_model = load_model('users.refresh_tokens.model', RefreshToken)

        refresh_token = refresh_token_model(
            token=random_string(
                prefix=cls.constants.TOKEN_PREFIX,
                length=cls.constants.TOKEN_LENGTH,
            ),
            status=status,
            user_id=user.object_id,
        )

        await refresh_token.save()

        return refresh_token



from eyja.hubs.config_hub import ConfigHub

from eyja_fastapi.errors import *
from eyja_fastapi.operators import (
    UserOperator,
    RefreshTokenOperator,
    AccessTokenOperator,
)


class AuthorizationProcess:
    login_field: str

    @classmethod
    def init(cls):
        cls.login_field = ConfigHub.get('users.login_field', 'email')

    @classmethod
    async def run(cls, **params):
        cls.init()

        login_field_value = params.get(cls.login_field, None)
        password = params.get('password', None)

        if not login_field_value:
            raise MissingRequiredFieldError(
                f'Required field "{cls.login_field}" is missing'
            )

        if not password:
            raise MissingRequiredFieldError(
                f'Required field "password" is missing'
            )

        user = await UserOperator.authenticate(login_field_value, password)
        if not user:
            raise UserNotFoundError(
                f'User is not found'
            )

        refresh_token = await RefreshTokenOperator.create_token(user)
        access_token = await AccessTokenOperator.create_token(refresh_token)

        return [user, refresh_token, access_token]



from typing import List, Type

from eyja.interfaces.db import (
    BaseStorageModel,
    DataFilter,
)
from eyja.errors import ObjectNotFoundError


async def get_for_user(
    model_cls: Type[BaseStorageModel],
    object_id: str,
    user: BaseStorageModel
) -> BaseStorageModel:
    items = await model_cls.find({
        'object_id': object_id,
        'user_id': user.object_id,
    })

    if len(items) < 1:
        raise ObjectNotFoundError(
            message=object_id,
        )

    return items[0]

async def find_for_user(
    model_cls: Type[BaseStorageModel],
    filter: DataFilter,
    user: BaseStorageModel
) -> List[BaseStorageModel]:
    filter.fields['user_id'] = user.object_id

    return await model_cls.find(filter)



from eyja.interfaces.db import DataFilter
from eyja.hubs.config_hub import ConfigHub
from eyja.utils import load_model

from eyja_fastapi.models import User


class UserOperator:
    @classmethod
    async def create_user(cls, password:str, **params):
        user_model = load_model('users.model', User)

        params.setdefault('is_active', False)
        params.setdefault('is_admin', False)

        user = user_model(**params)
        user.set_password(password)
        await user.save()

        return user

    @classmethod
    async def create_admin(cls, password:str, **params):
        params['is_active'] = True
        params['is_admin'] = True

        return await cls.create_user(password, **params)

    @classmethod
    async def authenticate(cls, login, password):
        login_field = ConfigHub.get('users.login_field', 'email')
        user_model = load_model('users.model', User)

        filter = DataFilter(
            fields={
                login_field: login,
                'is_active': True,
            }
        )

        users = await user_model.find(filter)
        if len(users) > 0 and users[0].check_password(password):
            return users[0]

        return None

    @classmethod
    async def get_user(cls, object_id: str):
        user_model = load_model('users.model', User)

        return await user_model.get(object_id)



from eyja.interfaces.db import BaseStorageModel
from eyja.utils import load_model

from eyja_fastapi.errors import *
from eyja_fastapi.models import RefreshToken
from eyja_fastapi.operators import (
    RefreshTokenOperator,
    AccessTokenOperator,
)


class RefreshProcess:
    refresh_token_model: BaseStorageModel

    @classmethod
    def init(cls):
        cls.refresh_token_model = load_model('users.refresh_tokens.model', RefreshToken)

    @classmethod
    async def run(cls, **params):
        cls.init()

        refresh_token_header = params.get('refresh_token', None)
        refresh_tokens = await cls.refresh_token_model.find({
            'token': refresh_token_header,
            'status': RefreshTokenOperator.statuses.ACTIVE,
        })
        if len(refresh_tokens) < 1:
            raise UserNotFoundError(
                f'Refresh token is not found'
            )

        return await AccessTokenOperator.create_token(refresh_tokens[0])



from fastapi import HTTPException, Request

from eyja_fastapi.processes.auth import AuthorizationProcess

from .router import users_router


@users_router.post('/login', name='login_user')
async def login_user(request: Request):
    json_data = await request.json()

    try:
        user, refresh_token, access_token = await AuthorizationProcess.run(**json_data)
    except Exception as ex:
        raise HTTPException(
            status_code=400,
            detail=str(ex),
        )

    return {
        'user': user.cleared_data,
        'access_token': access_token.token,
        'refresh_token': refresh_token.token,
    }



from .authorization import AuthorizationProcess
from .confirm import ConfirmProcess
from .refresh import RefreshProcess
from .registration import RegistrationProcess
from .unauthorization import UnauthorizationProcess


__all__ = [
    'AuthorizationProcess',
    'ConfirmProcess',
    'RefreshProcess',
    'RegistrationProcess',
    'UnauthorizationProcess',
]



from eyja.interfaces.db import BaseStorageModel
from eyja.hubs.config_hub import ConfigHub
from eyja.utils import load_model

from eyja_email.operators import EmailOperator

from eyja_fastapi.errors import *
from eyja_fastapi.models import User
from eyja_fastapi.operators import (
    UserOperator,
    ConfirmTokenOperator,
)


class RegistrationProcess:
    login_field: str
    confirm_registration: bool
    use_email: bool
    user_model: BaseStorageModel

    @classmethod
    def init(cls):
        cls.login_field = ConfigHub.get('users.login_field', 'email')
        cls.confirm_registration = ConfigHub.get('users.confirm_registration', True)
        cls.use_email = ConfigHub.get('users.use_email', True)
        cls.user_model = load_model('users.model', User)

    @classmethod
    async def send_email(cls, user):
        if cls.use_email:
            if cls.confirm_registration:
                confirm_token = await ConfirmTokenOperator.create_token(
                    ConfirmTokenOperator.types.AUTH,
                    user,
                )

                email = await EmailOperator.create(
                    subject=ConfigHub.get('users.email.registration.subject','Successful registration'),
                    sender=ConfigHub.get('users.email.sender'),
                    sender_name=ConfigHub.get('users.email.sender_name'),
                    recipient=user.email,
                    template=ConfigHub.get('users.email.registration.template', 'registration.j2'),
                    message_data={
                        'confirm_token': confirm_token,
                        'user': user,
                    }
                )
            else:
                email = await EmailOperator.create(
                    subject=ConfigHub.get('users.email.registration.subject','Successful registration'),
                    sender=ConfigHub.get('users.email.sender'),
                    sender_name=ConfigHub.get('users.email.sender_name'),
                    recipient=user.email,
                    template=ConfigHub.get('users.email.registration.template', 'registration.j2'),
                    message_data={
                        'user': user,
                    }
                )

            await EmailOperator.send(email)

    @classmethod
    async def run(cls, **params):
        cls.init()

        login_field_value = params.get(cls.login_field, None)
        if not login_field_value:
            raise MissingRequiredFieldError(
                f'Required field "{cls.login_field}" is missing'
            )

        if not params.get('password', None):
            raise MissingRequiredFieldError(
                f'Required field "password" is missing'
            )

        exists_users = await cls.user_model.find({cls.login_field: login_field_value})
        if len(exists_users) > 0:
            raise UserAlreadyExistsError(
                f'User "{login_field_value}" already exists'
            )

        params['is_active'] = not cls.confirm_registration
        params['is_admin'] = False

        user = await UserOperator.create_user(**params)

        await cls.send_email(user)

        return user



from fastapi import Request, HTTPException


def login_required(f):
    async def wrapper(request: Request):
        allow = all((
            'current_user' in request.scope,
            getattr(request.scope.get('current_user'), 'is_active', False),
        ))
        if not allow:
            raise HTTPException(
                status_code=401,
                detail='ACCESS_DENIED',
            )
        return await f(request)
    
    return wrapper

def admin_required(f):
    async def wrapper(request: Request):
        allow = all((
            'current_user' in request.scope,
            getattr(request.scope.get('current_user'), 'is_active', False),
            getattr(request.scope.get('current_user'), 'is_admin', False),
        ))
        if not allow:
            raise HTTPException(
                status_code=401,
                detail='ACCESS_DENIED',
            )
        return await f(request)
    
    return wrapper


from .misc_errors import MissingRequiredFieldError
from .user_errors import (
    UserAlreadyExistsError,
    UserNotFoundError
)


__all__ = [
    'MissingRequiredFieldError',
    'UserAlreadyExistsError',
    'UserNotFoundError',
]



import aioredis
import json
import html_to_json

from unittest import IsolatedAsyncioTestCase

from rethinkdb import r

from eyja.main import Eyja
from eyja.utils import render_template

from eyja_email.models import Email

from eyja_fastapi.processes.auth import RegistrationProcess
from eyja_fastapi.models import User, ConfirmToken
from eyja_fastapi.errors import *


class UserRegistrationTest(IsolatedAsyncioTestCase):
    redis_url = 'redis://localhost:30002/4'

    async def test_registration_wrong_username_field(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config.yml')

        with self.assertRaises(MissingRequiredFieldError):
            await RegistrationProcess.run(
                username='test0@test.com',
                password='passTeSt123'
            )

    async def test_registration_duplicate_user(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config.yml')

        await RegistrationProcess.run(
            email='test0@test.com',
            password='passTeSt123'
        )

        with self.assertRaises(UserAlreadyExistsError):
            await RegistrationProcess.run(
                email='test0@test.com',
                password='passTeSt1234'
            )

    async def test_registration_check_user(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config.yml')

        await RegistrationProcess.run(
            email='test1@test.com',
            password='passTeSt123'
        )

        finded_users = await User.find({'email':'test1@test.com'})
        self.assertEqual(len(finded_users), 1)
        self.assertFalse(finded_users[0].check_password('passTeSt1234'))
        self.assertTrue(finded_users[0].check_password('passTeSt123'))

    async def test_registration_check_confirm_token(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config.yml')

        user = await RegistrationProcess.run(
            email='test2@test.com',
            password='passTeSt123'
        )

        confirm_tokens = await ConfirmToken.find({'user_id': user.object_id})
        self.assertEqual(len(confirm_tokens), 1)

    async def test_registration_check_email(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config.yml')

        user = await RegistrationProcess.run(
            email='test3@test.com',
            password='passTeSt123'
        )

        emails = await Email.find({'recipient': user.email})
        self.assertEqual(len(emails), 1)

        redis_connection = aioredis.from_url(self.redis_url)
        async with redis_connection.client() as conn:
            redis_data = await conn.get(emails[0].object_id)

        confirm_tokens = await ConfirmToken.find({'user_id': user.object_id})

        rendered_template = await render_template(
            template_root='tests/templates',
            template='confirm_registration.j2',
            data={
                'user': user,
                'confirm_token': confirm_tokens[0],
            }
        )

        message = json.loads(redis_data.decode())
        self.assertEqual(message['message']['subject'], 'Test registration subject')
        self.assertEqual(message['message']['body']['text'], 'Test registration subject')
        self.assertEqual(
            html_to_json.convert(message['message']['body']['html']), 
            html_to_json.convert(rendered_template),
        )

    async def test_registration_check_email_without_confirmation(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config_without_confirmation.yml')

        user = await RegistrationProcess.run(
            email='test4@test.com',
            password='passTeSt123'
        )

        emails = await Email.find({'recipient': user.email})
        self.assertEqual(len(emails), 1)

        redis_connection = aioredis.from_url(self.redis_url)
        async with redis_connection.client() as conn:
            redis_data = await conn.get(emails[0].object_id)

        rendered_template = await render_template(
            template_root='tests/templates',
            template='success_registration.j2',
            data={
                'user': user,
            }
        )

        message = json.loads(redis_data.decode())
        self.assertEqual(message['message']['subject'], 'Success registration subject')
        self.assertEqual(message['message']['body']['text'], 'Success registration subject')
        self.assertEqual(
            html_to_json.convert(message['message']['body']['html']), 
            html_to_json.convert(rendered_template),
        )



from eyja.interfaces.db import DataFilter
from eyja.utils import random_string, load_model

from eyja_fastapi.models import ConfirmToken, User

from .types import ConfirmTokenTypes
from .constants import ConfirmTokenConstants


class ConfirmTokenOperator:
    types = ConfirmTokenTypes
    constants = ConfirmTokenConstants

    @classmethod
    async def create_token(cls, token_type, user):
        confirm_token_model = load_model('users.confirm_tokens.model', ConfirmToken)
        
        confirm_token = confirm_token_model(
            token=random_string(
                prefix=cls.constants.TOKEN_PREFIX,
                length=cls.constants.TOKEN_LENGTH,
            ),
            token_type=token_type,
            user_id=user.object_id,
        )

        await confirm_token.save()

        return confirm_token

    @classmethod
    async def verify_token(cls, token_type, token):
        confirm_token_model = load_model('users.confirm_tokens.model', ConfirmToken)
        user_model = load_model('users.model', User)

        filter = DataFilter(fields={
            'token': token,
            'token_type': token_type,
        })

        confirm_tokens = await confirm_token_model.find(filter)
        if len(confirm_tokens) < 1:
            return None

        await confirm_tokens[0].delete()

        return await user_model.get(confirm_tokens[0].user_id)



from eyja.interfaces.operators import BaseModelOperator
from eyja.interfaces.db import BaseStorageModel

from eyja_fastapi.helpers import (
    get_for_user,
    find_for_user,
)


class AuthorizedModelOperator(BaseModelOperator):
    @classmethod
    async def get_for_user(cls, object_id: str, user: BaseStorageModel):
        return await get_for_user(cls.model, object_id, user)

    @classmethod
    async def find_for_user(cls, filter, user: BaseStorageModel):
        return await find_for_user(cls.model, filter, user)



from typing import Optional

from pydantic import BaseModel

from passlib.handlers.pbkdf2 import pbkdf2_sha256


class UserMixin(BaseModel):
    password_hash: Optional[str]
    is_active: bool = False
    is_admin: bool = False

    def set_password(self, password):
        self.password_hash = pbkdf2_sha256.hash(password)

    def check_password(self, password):
        return pbkdf2_sha256.verify(password, self.password_hash)



from eyja.utils import random_string, load_model

from eyja_fastapi.models import AccessToken

from .constants import AccessTokenConstants


class AccessTokenOperator:
    constants = AccessTokenConstants

    @classmethod
    async def create_token(cls, refresh_token):
        access_token_model = load_model('users.access_tokens.model', AccessToken)

        access_token = access_token_model(
            token=random_string(
                prefix=cls.constants.TOKEN_PREFIX,
                length=cls.constants.TOKEN_LENGTH,
            ),
            refresh_token_id=refresh_token.object_id,
            user_id=refresh_token.user_id,
        )

        await access_token.save()

        return access_token



from fastapi import HTTPException, Request

from eyja_fastapi.decorators import login_required

from .router import users_router


@users_router.get('/current', name='current_user')
@login_required
async def current_user(request: Request):
    return {
        'user': request.scope['current_user'].cleared_data,
    }



from eyja.hubs.config_hub import ConfigHub


class RefreshTokenConstants:
    TOKEN_LENGTH = ConfigHub.get('users.refresh_tokens.token_length', 64)
    TOKEN_PREFIX = ConfigHub.get('users.refresh_tokens.token_prefix', 'rt2')



from eyja.interfaces.db import BaseStorageModel
from eyja.hubs.config_hub import ConfigHub


class ConfirmToken(BaseStorageModel):
    _namespace = ConfigHub.get('users.confirm_tokens.namespace', ':::confirm_tokens')
    _key_template = ConfigHub.get('users.confirm_tokens.key_template', 'token.token_type.user_id')
    _expiration = ConfigHub.get('users.confirm_tokens.expiration', 3*24*60*60)

    token: str
    token_type: str
    user_id: str



from eyja.interfaces.db import BaseStorageModel
from eyja.hubs.config_hub import ConfigHub

from .user_mixin import UserMixin


class User(UserMixin, BaseStorageModel):
    _namespace = ConfigHub.get('users.namespace', ':::users')
    _indexes = [
        'email'
    ] + BaseStorageModel._indexes
    _hidden_fields = [
        'password_hash'
    ]

    email: str




from eyja.interfaces.db import BaseStorageModel
from eyja.hubs.config_hub import ConfigHub


class RefreshToken(BaseStorageModel):
    _namespace = ConfigHub.get('users.refresh_tokens.namespace', ':::refresh_tokens')
    _key_template = ConfigHub.get('users.refresh_tokens.key_template', 'token.user_id.status')
    _expiration = ConfigHub.get('users.refresh_tokens.expiration', 3*24*60*60)

    token: str
    status: str
    user_id: str



from .router import users_router
from .confirm import confirm_user
from .current import current_user
from .login import login_user
from .logout import logout_user
from .registration import registration_user


__all__ = [
    'users_router',
    'confirm_user',
    'current_user',
    'login_user',
    'logout_user',
    'registration_user',
]



from fastapi import APIRouter, Request

from eyja.interfaces.operators import (
    create_model_operator,
    ModelOperator,
)
from eyja.interfaces.db import (
    BaseStorageModel
)

from eyja_fastapi.decorators import admin_required, login_required


def set_auth_mode(auth: str, f):
    if auth == 'admin':
        f = admin_required(f)
    elif auth == 'user':
        f = login_required(f)

    return f

def model_api(model_cls: BaseStorageModel, operator: ModelOperator = None, auth: dict = {}) -> APIRouter:
    auth.setdefault('get', 'admin')
    auth.setdefault('count', 'admin')
    auth.setdefault('create', 'admin')
    auth.setdefault('delete', 'admin')
    auth.setdefault('find', 'admin')
    auth.setdefault('update', 'admin')

    router = APIRouter()
    if not operator:
        operator = create_model_operator(model_cls)

    async def get(request: Request):
        data = await operator.get(request.path_params.get('_id'))
        return data

    async def count(request: Request):
        json_data = await request.json()
        return await operator.count(json_data)

    async def create(request: Request):
        json_data = await request.json()
        return await operator.create(json_data)

    async def delete(request: Request):
        _id: str = request.path_params.get('_id')
        await operator.delete(_id)
        return {}

    async def find(request: Request):
        json_data = await request.json()
        return await operator.find(json_data)

    async def update(request: Request):
        json_data = await request.json()
        return await operator.update(
            request.path_params.get('_id'),
            json_data,
        )

    all_auth = auth.get('all')
    get_auth = auth.get('get') if not all_auth else all_auth
    count_auth = auth.get('count') if not all_auth else all_auth
    create_auth = auth.get('create') if not all_auth else all_auth
    delete_auth = auth.get('delete') if not all_auth else all_auth
    find_auth = auth.get('find') if not all_auth else all_auth
    update_auth = auth.get('update') if not all_auth else all_auth
    
    router.get('/{_id}')(set_auth_mode(get_auth, get))
    router.post('/count')(set_auth_mode(count_auth, count))
    router.post('/')(set_auth_mode(create_auth, create))
    router.delete('/{_id}')(set_auth_mode(delete_auth, delete))
    router.post('/find')(set_auth_mode(find_auth, find))
    router.patch('/{_id}')(set_auth_mode(update_auth, update))

    return router



from eyja.interfaces.db import BaseStorageModel
from eyja.utils import load_model

from eyja_fastapi.errors import *
from eyja_fastapi.models import (
    User,
    ConfirmToken,
)
from eyja_fastapi.operators import ConfirmTokenOperator


class ConfirmProcess:
    user_model: BaseStorageModel
    confirm_token_model: BaseStorageModel

    @classmethod
    def init(cls):
        cls.user_model = load_model('users.model', User)
        cls.confirm_token_model = load_model('users.confirm_tokens.model', ConfirmToken)

    @classmethod
    async def run(cls, **params):
        cls.init()

        confirm_token_header = params.get('confirm_token', None)
        confirm_tokens = await cls.confirm_token_model.find({
            'token': confirm_token_header,
            'token_type': ConfirmTokenOperator.types.AUTH,
        })

        if len(confirm_tokens) < 1:
            raise UserNotFoundError(
                f'Confirm token is not found'
            )

        user = await cls.user_model.get(confirm_tokens[0].user_id)
        user.is_active = True
        await user.save()

        await confirm_tokens[0].delete()

        return user



from eyja.interfaces.db import BaseStorageModel
from eyja.hubs.config_hub import ConfigHub
from eyja.utils import load_model

from eyja_email.operators import EmailOperator

from eyja_fastapi.errors import *
from eyja_fastapi.models import (
    User,
    AccessToken,
    RefreshToken,
    ConfirmToken,
)
from eyja_fastapi.operators import (
    UserOperator,
    ConfirmTokenOperator,
    RefreshTokenOperator,
    AccessTokenOperator,
)


class UnauthorizationProcess:
    access_token_model: BaseStorageModel
    refresh_token_model: BaseStorageModel

    @classmethod
    def init(cls):
        cls.access_token_model = load_model('users.access_tokens.model', AccessToken)
        cls.refresh_token_model = load_model('users.refresh_tokens.model', RefreshToken)

    @classmethod
    async def run(cls, **params):
        cls.init()

        access_token = params.get('access_token')

        refresh_tokens = await cls.refresh_token_model.find({
            'object_id': access_token.refresh_token_id,
            'status': RefreshTokenOperator.statuses.ACTIVE,
        })
        if len(refresh_tokens) < 1:
            raise UserNotFoundError(
                f'Refresh token is not found'
            )

        refresh_tokens[0].status = RefreshTokenOperator.statuses.CANCELED
        await refresh_tokens[0].save()

        await cls.access_token_model.delete_all({
            'refresh_token_id': refresh_tokens[0].object_id,
        })



from fastapi import HTTPException, Request

from eyja_fastapi.processes.auth import RegistrationProcess

from .router import users_router


@users_router.post('/registration', name='registration_user')
async def registration_user(request: Request):
    json_data = await request.json()

    try:
        user = await RegistrationProcess.run(**json_data)
    except Exception as ex:
        raise HTTPException(
            status_code=400,
            detail=str(ex),
        )

    return {
        'user': user.cleared_data,
    }



from eyja.hubs.config_hub import ConfigHub


class AccessTokenConstants:
    TOKEN_LENGTH = ConfigHub.get('users.access_tokens.token_length', 32)
    TOKEN_PREFIX = ConfigHub.get('users.access_tokens.token_prefix', 'at2')



from fastapi import HTTPException, Request

from eyja_fastapi.decorators import login_required
from eyja_fastapi.processes.auth import UnauthorizationProcess

from .router import users_router


@users_router.get('/logout', name='logout_user')
@login_required
async def logout_user(request):
    await UnauthorizationProcess.run(
        access_token=request.scope['access_token'],
    )

    return {}



from .user import UserOperator
from .refresh_token import RefreshTokenOperator
from .confirm_token import ConfirmTokenOperator
from .access_token import AccessTokenOperator


__all__ = [
    'UserOperator',
    'RefreshTokenOperator',
    'ConfirmTokenOperator',
    'AccessTokenOperator',
]



from eyja.interfaces.db import BaseStorageModel
from eyja.hubs.config_hub import ConfigHub


class AccessToken(BaseStorageModel):
    _namespace = ConfigHub.get('users.access_tokens.namespace', ':::access_tokens')
    _key_template = ConfigHub.get('users.access_tokens.key_template', 'token.refresh_token_id.user_id')
    _expiration = ConfigHub.get('users.access_tokens.expiration', 3*24*60*60)

    token: str
    refresh_token_id: str
    user_id: str



from eyja.hubs.config_hub import ConfigHub


class ConfirmTokenConstants:
    TOKEN_LENGTH = ConfigHub.get('users.confirm_tokens.token_length', 32)
    TOKEN_PREFIX = ConfigHub.get('users.confirm_tokens.token_prefix', 'ct2')



from .access_token import AccessToken
from .confirm_token import ConfirmToken
from .refresh_token import RefreshToken
from .user import User
from .user_mixin import UserMixin


__all__ = [
    'AccessToken',
    'ConfirmToken',
    'RefreshToken',
    'User',
    'UserMixin',
]



from fastapi import Request

from eyja.utils import load_model

from eyja_fastapi.processes import auth as auth_processes
from eyja_fastapi.operators import (
    RefreshTokenOperator,
    UserOperator,
)
from eyja_fastapi.models import (
    AccessToken,
    RefreshToken,
)


async def auth_middleware(request: Request, call_next):
    refreshed_token = False
    access_token_header = request.headers.get('x-access-token')
    refresh_token_header = request.headers.get('x-refresh-token')

    access_token_model = load_model('users.access_tokens.model', AccessToken)
    refresh_token_model = load_model('users.refresh_tokens.model', RefreshToken)

    if access_token_header:
        access_tokens = await access_token_model.find({
            'token': access_token_header
        })
        access_token = None

        if len(access_tokens) < 1:
            refresh_tokens = await refresh_token_model.find({
                'token': refresh_token_header,
                'status': RefreshTokenOperator.statuses.ACTIVE,
            })

            if len(refresh_tokens) > 0:
                access_token = await auth_processes.RefreshProcess.run(
                    refresh_token=refresh_token_header,
                )
                refreshed_token = True
        else:
            access_token = access_tokens[0]

        if access_token:
            request.scope['current_user'] = await UserOperator.get_user(
                access_token.user_id
            )
            request.scope['access_token'] = access_token

    response = await call_next(request)

    if refreshed_token:
        response.headers["x-new-access-token"] = access_token.token

    return response



import aioredis
import json
import html_to_json

from unittest import IsolatedAsyncioTestCase

from rethinkdb import r

from eyja.main import Eyja
from eyja.utils import render_template

from eyja_email.models import Email

from eyja_fastapi.processes.auth import (
    AuthorizationProcess,
    RegistrationProcess,
)
from eyja_fastapi.models import User, ConfirmToken
from eyja_fastapi.errors import *


class UserAuthorizationTest(IsolatedAsyncioTestCase):
    redis_url = 'redis://localhost:30002/4'

    async def test_authorization_wrong_username_field(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config_without_confirmation.yml')

        await RegistrationProcess.run(
            email='test10@test.com',
            password='passTeSt123'
        )

        with self.assertRaises(MissingRequiredFieldError):
            await AuthorizationProcess.run(
                username='test10@test.com',
                password='passTeSt123'
            )

    async def test_authorization_wrong_user(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config_without_confirmation.yml')

        await RegistrationProcess.run(
            email='test11@test.com',
            password='passTeSt123'
        )

        with self.assertRaises(UserNotFoundError):
            await AuthorizationProcess.run(
                email='test111@test.com',
                password='passTeSt123'
            )

    async def test_authorization_wrong_password(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config_without_confirmation.yml')

        await RegistrationProcess.run(
            email='test12@test.com',
            password='passTeSt123'
        )

        with self.assertRaises(UserNotFoundError):
            await AuthorizationProcess.run(
                email='test12@test.com',
                password='passTeSt1234'
            )

    async def test_authorization_inactive_user(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config.yml')

        await RegistrationProcess.run(
            email='test13@test.com',
            password='passTeSt123'
        )

        with self.assertRaises(UserNotFoundError):
            await AuthorizationProcess.run(
                email='test13@test.com',
                password='passTeSt1234'
            )

    async def test_authorization(self):
        await Eyja.reset()
        await Eyja.init(config_file='tests/config/full_config_without_confirmation.yml')

        user = await RegistrationProcess.run(
            email='test14@test.com',
            password='passTeSt123'
        )

        auth_user, refresh_token, access_token = await AuthorizationProcess.run(
            email='test14@test.com',
            password='passTeSt123',
        )

        self.assertEqual(auth_user.email, user.email)
        self.assertEqual(refresh_token.user_id, user.object_id)
        self.assertEqual(access_token.user_id, user.object_id)
